/*
 * Copyright (C) 2005 by egnite Software GmbH
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/*!
 * \file serwin.c
 * \brief Serial port I/O routines for Win32.
 *
 * \verbatim
 *
 * $Log: serwin.c,v $
 * Revision 1.2  2006/03/21 11:04:20  haraldkipp
 * Fixed type mismatching.
 *
 * Revision 1.1  2005/11/25 09:06:48  haraldkipp
 * First check-in
 *
 *
 * \endverbatim
 */

#include <stdio.h>
#include <windows.h>

#include "serport.h"

HANDLE hdev = INVALID_HANDLE_VALUE;

int SerialPortOpen(char *name)
{
    DCB dcb;
    COMMTIMEOUTS ctmo;

    hdev = CreateFile(name, GENERIC_READ | GENERIC_WRITE, 0, NULL, 
        OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
    if(hdev == INVALID_HANDLE_VALUE) {
        return -1;
    }
    dcb.DCBlength = sizeof(DCB);
    if(!GetCommState(hdev, &dcb)) {
    	CloseHandle(hdev);
	    return -1;
    }
	dcb.BaudRate = CBR_115200;
	dcb.fParity = FALSE;
	dcb.fOutxCtsFlow = FALSE;
	dcb.fOutxDsrFlow = FALSE;
	dcb.fDtrControl = DTR_CONTROL_ENABLE;
	dcb.fDsrSensitivity = FALSE;
	dcb.fOutX = FALSE;
	dcb.fInX = FALSE;
	dcb.fErrorChar = FALSE;
	dcb.fNull = FALSE;
	dcb.fRtsControl = RTS_CONTROL_HANDSHAKE;
	dcb.fAbortOnError = FALSE;
	dcb.ByteSize = 8;
	dcb.StopBits = ONESTOPBIT;

	if(!SetCommState(hdev, &dcb)) {
	    CloseHandle(hdev);
	    return -1;
	}

    ctmo.ReadIntervalTimeout = 10;
    ctmo.ReadTotalTimeoutMultiplier = 1;
    ctmo.ReadTotalTimeoutConstant = 10000;
    ctmo.WriteTotalTimeoutMultiplier = 1;
    ctmo.WriteTotalTimeoutConstant = 5000;

    SetCommTimeouts(hdev, &ctmo);

    return 0;
}

void SerialPortClose(void)
{
    if(hdev != INVALID_HANDLE_VALUE) {
        CloseHandle(hdev);
        hdev = INVALID_HANDLE_VALUE;
    }
}

/*!
 * \return Number of bytes received, 0 on timeout or -1 if an error occured.
 */
int SerialPortRead(char *buf, int siz)
{
    DWORD got;

    if(hdev == INVALID_HANDLE_VALUE) {
        return -1;
    }
    if(!ReadFile(hdev, buf, siz, &got, NULL)) {
        return -1;
    }
    return (int)got;
}

/*!
 * \return 0 if successful or -1 in case of an error.
 */
int SerialPortWrite(const char *buf, int len)
{
    DWORD written;

    if(hdev == INVALID_HANDLE_VALUE) {
        return -1;
    }
    if(!WriteFile(hdev, buf, len, &written, NULL) || written != (DWORD)len) {
        return -1;
    }
    return 0;
}

void SerialPortClear(void)
{
    DWORD e;
    COMSTAT s;
    DWORD got;
    char ch;
    DWORD i;

    if (ClearCommError(hdev, &e, &s)) {
        for (i = 0; i < s.cbInQue; i++) {
            ReadFile(hdev, &ch, 1, &got, NULL);
        }
    }
}
