/*
 * Copyright (C) 2005-2006 by egnite Software GmbH
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For additional information see http://jtagomat.sourceforge.net/
 */

#include "flash.h"
#include "host.h"

#include "trace.h"

unsigned long ReadIdentifier(void)
{
    flashptr_t base = (flashptr_t)HostRead();

    TraceLong((flashadr_t)base);

    return FlashIdentifier(base);
}

unsigned long EraseSector(void)
{
    flashptr_t base = (flashptr_t)HostRead();
    flashadr_t addr = (flashadr_t)HostRead();

    TraceLong((flashadr_t)base);
    TraceChar('@');
    TraceLong(addr);

    return FlashEraseSector(base, (flashptr_t)addr);
}

unsigned long ProgramData(void)
{
    unsigned long i;
    unsigned long data;
    unsigned long result = 0;
    flashptr_t base = (flashptr_t)HostRead();
    flashadr_t addr = (flashadr_t)HostRead();
    unsigned long len = HostRead();;

    TraceLong((flashadr_t)base);
    TraceChar('@');
    TraceLong(addr);
    TraceChar(',');
    TraceLong(len);

    for (i = 0; i < len; i += 4) {
        data = HostRead();
#ifdef FLASH_8BIT
        if (result == 0) {
            if (FlashWrite(base, (flashptr_t)(addr + i), (flashdat_t)data)) {
                result = 1;
            }
        }
        if (result == 0) {
            data >>= 8;
            if (FlashWrite(base, (flashptr_t)(addr + i + 1), (flashdat_t)data)) {
                result = 1;
            }
        }
        if (result == 0) {
            data >>= 8;
            if (FlashWrite(base, (flashptr_t)(addr + i + 2), (flashdat_t)data)) {
                result = 1;
            }
        }
        if (result == 0) {
            data >>= 8;
            if (FlashWrite(base, (flashptr_t)(addr + i + 3), (flashdat_t)data)) {
                result = 1;
            }
        }
#else
        if (result == 0) {
            if (FlashWrite(base, (flashptr_t)(addr + i), (flashdat_t)data)) {
                result = 1;
            }
        }
        if (result == 0) {
            data >>= 16;
            if (FlashWrite(base, (flashptr_t)(addr + i + 2), (flashdat_t)data)) {
                result = 1;
            }
        }
#endif
    }
    return result;
}

int main(void)
{
    unsigned long cmd;
    unsigned long result;

    TraceInit();
    TraceString("\n\nEthernut 3 BaseFlasher 0.0.8\n");

    for (;;) {
        /* Wait for host command. */
        cmd = HostRead();

        switch(cmd) {
        case 0xFFFFFF01:
            /* Set base address and get identifier */
            TraceString("ID(");
            result = ReadIdentifier();
            break;
        case 0xFFFFFF03:
            /* Erase device */
            TraceString("ED(");
            result = 1;
            break;
        case 0xFFFFFF05:
            /* Erase sector */
            TraceString("ES(");
            result = EraseSector();
            break;
        case 0xFFFFFF09:
            /* Program data */
            TraceString("PD(");
            result = ProgramData();
            break;
        default:
            TraceString("UN(");
            result = 1;
            break;
        }

        TraceString(")=");
        TraceLong(result);
        TraceString("\n");
        HostPut(result);
    }
}

void NutInit(void)
{
    main();
}
